.include "nes.inc"
.include "global.inc"

.import font_tophalves, font_bottomhalves

srclo = $0000
srchi = $0001
dstlo = $0002
dsthi = $0003
numlines = $0004

.segment "CODE"
.proc cls_puts_multiline
  pha
  tya
  pha

  lda #VBLANK_NMI
  sta PPUCTRL
  asl a
  sta PPUMASK

  ; Start by clearing the first nametable
  ldx #$20
  ;lda #$00
  tay
  jsr ppu_clear_nt

  TOPLEFT = $2002
  lda #<TOPLEFT
  sta dstlo
  lda #>TOPLEFT
  sta dsthi
  lda #13
  sta numlines
  pla
  tay
  pla
  ; fall through to puts_multiline_16
.endproc

;;
; Writes the string at (AAYY) to lines starting at 2 and 3.
; At finish, 0 and 1 points to start of last line, and Y is the
; length of the last line.
; @param AAYY pointer to text
; @param $02-$03 nametable destination pointer
; @param $04 maximum number of lines to write
.proc puts_multiline_16
  sta srchi
  sty srclo
.endproc
.proc puts_multiline_16_0
lineloop:
  lda dsthi
  ldx dstlo
  jsr puts_16
  lda dstlo
  clc
  adc #64
  sta dstlo
  bcc :+
  inc dsthi
:
  lda (srclo),y
  beq done
  dec numlines
  beq done
  tya
  sec  ; +1: also skip newline
  adc srclo
  sta srclo
  bcc lineloop
  inc srchi
  bcs lineloop
done:
  rts
.endproc

;;
; Writes the string at (0) to the nametable at AAXX.
; Does not write to memory.
.proc puts_16
  sta PPUADDR
  stx PPUADDR
  pha
  txa
  pha
  ldy #0
copyloop1:
  lda (0),y
  cmp #' '
  bcc after_copyloop1
  tax
  lda font_tophalves-' ',x
  sta PPUDATA
  iny
  bne copyloop1
after_copyloop1:
  
  pla
  clc
  adc #32
  tax
  pla
  adc #0
  sta PPUADDR
  stx PPUADDR
  ldy #0
copyloop2:
  lda (0),y
  cmp #' '
  bcc after_copyloop2
  tax
  lda font_bottomhalves-' ',x
  sta PPUDATA
  iny
  bne copyloop2
after_copyloop2:
  rts
.endproc

;;
; Writes spaced hex bytes (e.g. 09 F9 ) to the nametable.
; @param AAXX nametable destination
; @param Y number of bytes to dump (up to 8)
; @param $0004 pointer to start of bytes to dump
; @return $0002-$0003: nametable address at end of hex dump;
; $0004-$0005: unchanged; $0006-$000A: overwritten
.proc hexdump8
dstlo = 2
dsthi = 3
  stx dstlo
  sta dsthi
  ; falls through
.endproc

;;
; Writes spaced hex bytes (e.g. 09 F9 ) to the nametable.
; @param $0002 nametable destination (16-bit)
; @param Y number of bytes to dump (up to 8)
; @param $0004 pointer to start of bytes to dump
; @return $0002-$0003: nametable address at end of hex dump;
; $0004-$0005: unchanged; $0006-$000A: overwritten
.proc hexdump8_02
strlo = 0
strhi = 1
dstlo = hexdump8::dstlo
dsthi = hexdump8::dsthi
hexsrc = 4
hexoffset = 6
hexbuf = 7
bytesleft = 10
  sty bytesleft
  lda #0
  sta hexbuf+2
  sta strhi
  sta hexoffset
  lda #<hexbuf
  sta strlo
  ldy hexoffset
loop:
  lda (hexsrc),y
  lsr a
  lsr a
  lsr a
  lsr a
  jsr hdig
  sta hexbuf+0
  lda (hexsrc),y
  iny
  sty hexoffset
  and #$0F
  jsr hdig
  sta hexbuf+1
  
  lda dstlo
  tax
  clc
  adc #3
  sta dstlo
  lda dsthi
  jsr puts_16
  ldy hexoffset
  cpy bytesleft
  bne loop
  rts
hdig:
  cmp #10
  bcc :+
  adc #'A'-'0'-11
:
  adc #'0'
  rts
.endproc

;;
; Calculates the nametable address of a line of text on the screen.
; @param A vertical position in 16-pixel increments
; @return high byte in $03, low byte in A
.proc seek_nt_line_y
  tya
.endproc
.proc seek_nt_line_a
dsthi = $03
  ora #$20 << 2
  lsr a
  sta dsthi
  lda #0
  ror a
  lsr dsthi
  ror a
  rts
.endproc
.import font_chr
.importzp FONT_NUM_TILES
.proc load_font
  lda #VBLANK_NMI
  sta PPUCTRL
  asl a
  sta PPUMASK
  sta PPUADDR
  sta PPUADDR
  lda #<font_chr
  sta ciSrc
  lda #>font_chr
  sta ciSrc+1
  ldx #FONT_NUM_TILES
  jmp unpb53_xtiles
.endproc

